# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-04-23 09:08
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('flow', '0035_process_entity_new'),
    ]

    operations = [
        migrations.AddField(
            model_name='data',
            name='collection',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='data2', to='flow.Collection'),
        ),
        migrations.AddField(
            model_name='data',
            name='entity2',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='data2', to='flow.Entity'),
        ),
        migrations.AddField(
            model_name='entity',
            name='collection',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='entities2', to='flow.Collection'),
        ),
    ]
