# Generated by Django 2.2.1 on 2019-06-18 14:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('flow', '0034_callable_defaults'),
    ]

    operations = [
        migrations.AddField(
            model_name='process',
            name='entity_always_create',
            field=models.BooleanField(default=False),
        ),
    ]
